package com.flyfish.framework.controller;

import com.flyfish.framework.annotations.Operation;
import com.flyfish.framework.bean.Result;
import com.flyfish.framework.configuration.annotations.CurrentUser;
import com.flyfish.framework.domain.UserPasswordDto;
import com.flyfish.framework.domain.UserQo;
import com.flyfish.framework.domain.base.IUser;
import com.flyfish.framework.domain.po.Role;
import com.flyfish.framework.domain.po.User;
import com.flyfish.framework.enums.UserStatus;
import com.flyfish.framework.service.UserService;
import com.flyfish.framework.utils.Assert;
import com.flyfish.framework.utils.StrengthUtils;
import org.springframework.security.core.context.ReactiveSecurityContextHolder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Mono;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.Optional;

@RestController
@RequestMapping("/users")
public class UserController extends BaseController<User, UserQo> {

    @Resource
    private PasswordEncoder passwordEncoder;

    /**
     * 重置密码逻辑
     *
     * @param body 请求体
     * @param user 用户
     * @return 结果
     */
    @PutMapping("{id}/passwords")
    @Operation("重置密码")
    public Result<Void> resetPassword(@PathVariable String id, @RequestBody User body, @CurrentUser User user) {
        Assert.hasText(body.getPassword(), "重置密码必需携带密码！");
        Assert.isTrue(Optional.ofNullable(user.getRoles()).map(roles -> roles.stream().anyMatch(Role::getAdmin))
                .orElse(false), "您没有管理员权限，无法重置密码！");
        Assert.isTrue(StrengthUtils.isValid(body.getPassword()));
        User updating = new User();
        updating.setId(id);
        updating.setStatus(UserStatus.NORMAL);
        updating.setErrorCount(0);
        updating.setPassword(passwordEncoder.encode(body.getPassword()));
        service.updateSelectiveById(updating);
        return Result.ok();
    }

    /**
     * 修改密码逻辑
     *
     * @param passwordDto 密码dto
     * @param user        用户
     * @return 结果
     */
    @PutMapping("/passwords")
    @Operation("修改密码")
    public Result<Void> changePassword(@Valid @RequestBody UserPasswordDto passwordDto, @CurrentUser User user) {
        // 检查原密码
        Assert.isTrue(passwordEncoder.matches(passwordDto.getOldPassword(), user.getPassword()), "原密码不正确！");
        Assert.isTrue(!passwordEncoder.matches(passwordDto.getPassword(), user.getPassword()), "新密码和旧密码一致，输入个新的吧！");
        userContext.setUser(user);
        // 更新密码
        User updating = new User();
        updating.setId(user.getId());
        updating.setPassword(passwordEncoder.encode(passwordDto.getPassword()));
        service.updateSelectiveById(updating);
        return Result.ok();
    }

    /**
     * 获取当前用户
     *
     * @return 结果
     */
    @GetMapping("/current")
    public Mono<Result<IUser>> getCurrentUser() {
        UserService userService = getService();
        return ReactiveSecurityContextHolder.getContext()
                .map(context -> (IUser) context.getAuthentication().getPrincipal())
                .map(Result::ok);
    }
}
