package com.flyfish.framework.beans.meta;

import com.flyfish.framework.bean.Result;
import com.flyfish.framework.beans.resolver.DynamicRestBeanResolver;
import com.flyfish.framework.domain.base.Vo;
import com.flyfish.framework.utils.StringFormats;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ClassUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;

/**
 * bean控制器，可以通过类直接返回bean信息
 *
 * @author wangyu
 */
@RestController
@RequestMapping("beans")
@Slf4j
public class BeanController {

    @Resource
    private DynamicRestBeanResolver dynamicRestBeanResolver;

    /**
     * 通过bean名称获取
     *
     * @param name 名称
     * @return 结果
     */
    @GetMapping("{name}")
    public Result<BeanInfo> getByName(@PathVariable String name) {
        if (StringUtils.isBlank(name)) {
            return Result.empty();
        }
        Class<?> clazz = dynamicRestBeanResolver.getClass(name);
        if (null == clazz) {
            return Result.empty();
        }
        return Result.ok(getByClass(clazz));
    }

    /**
     * 通过类标识返回bean信息
     *
     * @return 结果
     */
    @GetMapping("")
    public Result<BeanInfo> beanInfo(String className) {
        if (StringUtils.isBlank(className)) {
            return Result.empty();
        }
        try {
            Class<?> clazz = ClassUtils.getClass(className);
            return Result.ok(getByClass(clazz));
        } catch (ClassNotFoundException e) {
            log.error(e.getMessage(), e);
            return Result.error(e.getMessage());
        }
    }

    private BeanInfo getByClass(Class<?> clazz) {
        BeanInfo info = new BeanInfo();
        if (clazz.isAnnotationPresent(RestBean.class)) {
            RestBean annotation = clazz.getAnnotation(RestBean.class);
            info.setName(annotation.name());
            info.setCode(annotation.value());
            info.setSearch(BeanProperty.from(annotation.queryClass()));
            if (!Vo.class.equals(annotation.listViewClass())) {
                info.setColumns(BeanProperty.from(annotation.listViewClass()));
            }
        } else {
            info.setCode(StringFormats.camel2Line(ClassUtils.getShortClassName(clazz)));
        }
        info.setProperties(BeanProperty.from(clazz));
        return info;
    }

    /**
     * 列出自动注册的bean，快速接入
     *
     * @return 结果
     */
    @GetMapping("list")
    public Result<List<String>> beanList() {
        return Result.accept(dynamicRestBeanResolver.getClasses().stream().map(ClassUtils::getCanonicalName)
                .collect(Collectors.toList()));
    }
}
